# frozen_string_literal: true

class Hermes::Api::Dmo::HousingRequestEntityFormLevelsController < Hermes::Api::Dmo::ApplicationController
  before_action :set_entity
  before_action :set_form_level, only: %i[show update destroy]

  VALID_OPERATORS = %w[$in $nin $ne $gt $gte $lt $lte $eq $regex $all]
  ACCEPTABLE_PARAMS = %w[
    consent_fields
    description
    level
    delegation
    entity_id
    created_at
    updated_at
  ]
  MUTATE_PARAMS = %i[
    consent_fields
    description
    level
    delegation
    entity_id
  ]
  mutate_params_only *MUTATE_PARAMS

  def index
    acceptable_params *ACCEPTABLE_PARAMS
    valid_operators *VALID_OPERATORS

    form_levels = @entity.form_levels.where(build_query(Hermes::HousingRequestEntityFormLevel)).order(order_params)
    paginate(
      form_levels,
      each_serializer: Hermes::HousingRequestEntityFormLevelSerializer
    )
  end

  def show
    return render_json_error(:unauthorized, :not_readers) unless @entity.readable_by?(@user)

    super
  end

  def create
    return render_json_error(:unauthorized, :not_entreprise) unless current_user_roles.include?('entreprise') || current_user_roles.include?('superadministrateur') || current_user_roles.include?('collaborateur entreprise')

    user_id = @user.id
    @instance = @entity.form_levels.build(mutate_valid_params.merge(user_id: user_id))
    return respond_with_errors(@instance) unless @instance.save

    render(jsonapi: @instance, serializer: Hermes::HousingRequestEntityFormLevelSerializer, status: :created)
  end

  private

  def set_form_level
    @instance = @entity.form_levels.find_by(id: params[:id])
  end

  def set_entity
    @entity = Hermes::Entity.readable_by(@user).find_by(id: params[:entity_id])
    return render_json_error(:not_found, :instance_not_found) unless @entity
  end
end
