module Hermes
  module RexClient
    class UpdateContactTask

      def update_contact(rex_entity, client = Hermes::RexClient::Client.new, yopmail = false, skip_update_groups = false)
        fields = rex_entity['fields']
        return if !fields['email'] 

        if Rails.env.test? || yopmail
          fields['email'] = fields['email'].split('@')[0] + '@yopmail.com'
        end

        contact = User.find_by(email: fields['email'].downcase)

        if !contact
          contact = User.create(user_fields(fields).merge(password: Devise.friendly_token[0,20]))
        else
          contact.update(user_fields(fields))
        end

        
        unless skip_update_groups
          contact.group_ids = []
          [
            {fieldName: 'id_etablissement_cristal', classname: Hermes::Entity },
            {fieldName: 'id_bailleur_cristal', classname: Hermes::Lessor },
          ].each do |type|
            query = fields[type[:fieldName]] if fields[type[:fieldName]].is_a? String
            query = {'$in': fields[type[:fieldName]]} if fields[type[:fieldName]].is_a? Array

            ids = classname.where(id_cristal: query).pluck(:manager_group_id) 

            contact.group_ids = (contact.group_ids + (ids || [])).uniq
          end
        end
        
        contact.save!
        contact
      end


      def user_fields(fields)
        mobileNumber = fields['tel_pro'].blank? ? '1234567890' : fields['tel_pro']
        role = fields['rolecrelocal'].blank? ? 'rce' : fields['rolecrelocal'].downcase
        {
          username: fields['email'].downcase,
          first_name: fields['prenom'],
          last_name: fields['nom'],
          email: fields['email'].downcase,
          login: fields['email'].downcase,
          auth_roles: [role],
          phone_number: mobileNumber,
          provider: HERMES_CONFIG['identity_provider_key'],
          create_user_data: {
            gender: 'M',
            firstName: fields['prenom'],
            lastName: fields['nom'],
            email: fields['email'].downcase,
            login: fields['email'].downcase,
            mobileNumber: mobileNumber,
            roles: [role]
          },
        }
      end
    end
  end
end
